<?php

require_once('/usr/local/www/berogui/includes/isgwtelnet.php');

class ShortMessageService {

	private		$_isgw,
			$_error,
			$_errorMessage;

	function __construct ()  {

		$this->_error = false;
		$this->_errorMessage = '';

		$this->_isgw = new isgwtelnet();
		if ($this->_isgw->isgw_login() == false) {
			$this->_error = true;
			$this->_errorMessage = 'Connection refused by ISGW';
		}

		foreach (array('in', 'out', 'failout') as $folder) {
			if (!file_exists('/tmp/sms/' . $folder)) {
				@exec('/bin/mkdir -p /tmp/sms/' . $folder);
			}
		}
	}

	public function getErrorText () {
		return(($this->_error = true) ? $this->_errorMessage : false);
	}

	private function _evaluateSmsStatus ($status) {
		switch ($status['status']) {
		case 'OK':
			return(array('smsId' => $status['smsId'], 'status' => $status['status']));
		case 'ERROR':
			return(array('smsId' => $status['smsId'], 'status' => $status['status'], 'message' => $status['message']));
		}

		return(array('smsId' => $status['smsId'], 'status' => 'UNKNOWN'));
	}

	public function sendMessage ($phoneNumber, $shortMessage, $gsmPort = 1, $confirmationWait = false) {

		if ($this->_error == true) {
			return(false);
		}

		if (empty($phoneNumber) || empty($shortMessage)) {
			$this->_error = true;
			$this->_errorMessage = 'Phone-Number or Message missing';
			return(false);
		}

		if (strlen($shortMessage) > 670) {
			$this->_error = true;
			$this->_errorMessage = 'Message too long';
			return(false);
		}

		// create sms-file
		$smsId = uniqid();
		$smsFile = '/tmp/sms/out/' . $smsId . '.sms';
		$smsContent = 'num=' . $phoneNumber . "\n" . 'port='. $gsmPort . "\n" . 'msg=' . stripslashes($shortMessage);

		// write sms and send it
		@file_put_contents($smsFile, $smsContent);
		$this->_isgw->ssms();

		if ($confirmationWait == false) {
			return($smsId);
		}

		for ($i = 0; $i < $confirmationWait; $i++) {
			sleep(1);
			$sendReport = @file('/tmp/sms/send_report');
			if (($sendReport == false) || (count($sendReport) == 0)) {
				continue;
			}

			foreach ($sendReport as $line) {
				unset($matchesStatus);
				unset($matchesMsg);

				preg_match('/(?<smsId>[0-9a-f]*):(?<status>[A-Z]*)/', $line, $matchesStatus);
				if ($matchesStatus['smsId'] != $smsId) {
					continue;
				}

				preg_match('/[0-9a-f]*:[A-Z]*:(?<message>.*)/', $line, $matchesMsg);
				return($this->_evaluateSmsStatus(array('smsId' => $matchesStatus['smsId'], 'status' => $matchesStatus['status'], 'message' => $matchesMsg['message'])));
			}

		}

		// if we get here, we had a timeout
		return(array('smsId' => $smsId, 'status' => 'TIMEOUT'));
	}

	public function listMessages ($queueName) {

		if (!in_array($queueName, array('in', 'out', 'failout'))) {
			return(null);
		}

		if (($dp = @opendir('/tmp/sms/' . $queueName)) == null) {
			return(null);
		}

		$ret = array();
		while (($file = readdir($dp)) !== false) {
			if (($file == '.') || ($file == '..')) {
				continue;
			}

			if ($queueName == 'failout') {
				$ret[] = $file;
				continue;
			}

			if (preg_match('/.*\.sms$/', $file)) {
				$ret[] = basename($file, '.sms');
			}
		}

		return($ret);
	}

	public function getMessage ($queueName, $smsId) {

		if (!in_array($queueName, array('in', 'out', 'failout'))) {
			return(null);
		}

		switch ($queueName) {
		case 'in':
			$smsFile = '/tmp/sms/' . $queueName . '/' . $smsId . '.sms';
			$ret['message'] = (!file_exists($smsFile)) ? 'Message not found' : @file_get_contents($smsFile);
			break;
		case 'out':
			$smsFile = '/tmp/sms/' . $queueName . '/' . $smsId . '.sms';
			$ret['message'] = (!file_exists($smsFile)) ? 'Message not found' : @file_get_contents($smsFile);
			$ret['log'] = (!file_exists($smsFile . '.log')) ? '' : @file_get_contents($smsFile . '.log');
			break;
		case 'failout':
			$smsFile = '/tmp/sms/' . $queueName . '/' . $smsId . '/message';
			$ret['message'] = (!file_exists($smsFile)) ? 'Message not found' : @file_get_contents($smsFile);
			$ret['log'] = (!file_exists($smsFile . '.log')) ? '' : @file_get_contents($smsFile . '.log');
			break;
		}

		return($ret);
	}

	public function deleteMessage ($queueName, $smsId) {

		if (!in_array($queueName, array('in', 'out', 'failout'))) {
			return;
		}

		switch ($queueName) {
		case 'in':
			@unlink('/tmp/sms/' . $queueName . '/' . $smsId . '.sms');
			break;
		case 'out':
			@unlink('/tmp/sms/' . $queueName . '/' . $smsId . '.sms');
			@unlink('/tmp/sms/' . $queueName . '/' . $smsId . '.sms.log');
			break;
		case 'failout':
			@unlink('/tmp/sms/' . $queueName . '/' . $smsId . '/message');
			@unlink('/tmp/sms/' . $queueName . '/' . $smsId . '/message.log');
			@rmdir('/tmp/sms/' . $queueName . '/' . $smsId);
			break;
		}
	}

	public function getSendReport () {

		if (!file_exists('/tmp/sms/send_report')) {
			return(false);
		}

		return(file_get_contents('/tmp/sms/send_report'));
	}

	public function getDeliveryReport () {
	    
	    if (!file_exists('/tmp/sms/delivery_report')) {
	        return(false);
	    }
	    
	    return(file_get_contents('/tmp/sms/delivery_report'));
	}
	
	public function purgeSendReport () {
		@unlink('/tmp/sms/send_report');
	}

	public function getMessageError ($smsId) {

		if (!file_exists('/tmp/sms/send_report')) {
			return(false);
		}

		$lines = @file('/tmp/sms/send_report');
		foreach ($lines as $line) {
			unset($matches);
			preg_match('/(?<smsId>[0-9a-f]*):(?<status>[A-Z]*):(?<message>.*)/', $line, $matches);
			if ($matches['smsId'] == $smsId) {
				break;
			}
		}

		return($matches['message']);
	}
}
?>
